from ..pyalexaskill import AlexaBaseHandler


class AlexaDeploymentHandler(AlexaBaseHandler):
    """
    Sample concrete implementation of the AlexaBaseHandler to test the
    deployment scripts and process.
    All on_ handlers call the same test response changing the request type
    spoken.
    """

    def __init__(self):
        super(self.__class__, self).__init__()

    def _test_response(self, msg):
        session_attributes = {}
        card_title = "Test Response"
        card_output = "Test card output"
        speech_output = "Welcome to the Python Alexa Test Deployment for request type {0}.  It seems to have worked".format(msg)
        # If the user either does not reply to the welcome message or says something
        # that is not understood, they will be prompted again with this text.
        reprompt_text = "Reprompt text for the Alexa Test Deployment"
        should_end_session = True

        speechlet = self._build_speechlet_response(card_title, card_output, speech_output, reprompt_text, should_end_session)

        return self._build_response(session_attributes, speechlet)

    def on_processing_error(self, event, context, exc):
        return self._test_response("on processing error")

    def on_launch(self, launch_request, session):
        return self._test_response("on launch")

    def on_session_started(self, session_started_request, session):
        return self._test_response("on session started")

    def on_intent(self, intent_request, session):
        return self._test_response("on intent")

    def on_session_ended(self, session_end_request, session):
        return self._test_response("on session end")




#!/usr/bin/env python
import os
import sys
import getopt

"""
Script will create a test intent schema and sample utterance file..

"""


intent_template = """{
  "intents": [
    {
      "intent": "DeploymentIntent",
      "slots": []
    }
  ]
}
"""

utterance_template = """DeploymentIntent what is the deployment status
DeploymentIntent what is the status
DeploymentIntent provide the deployment status
"""

def _mkdirp(directory):
    if not os.path.isdir(directory):
        os.makedirs(directory)


def main(argv):
    root_project_dir=None

    try:
        opts, args = getopt.getopt(argv, "hr:", ["root="])
    except getopt.GetoptError:
        print 'create_alexa_test_skills.py -r <root project dir>'
        print 'if -r option not supplied it will look for PWD environment variable'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'create_alexa_test_skills.py -r <root project dir>'
            print 'if -r option not supplied it will look for PWD environment variable'
            print '<include files> are relative to root project dir'
            sys.exit()
        elif opt in ("-r", "--root"):
            root_project_dir = arg

    if not root_project_dir:
        root_project_dir = os.environ.get("PWD")
        if root_project_dir is None:
            raise ValueError("Must supply -r or --root option")


    root_deployments_dir = "{0}/alexa_skills/deployment_test2".format(root_project_dir)
    _mkdirp(root_deployments_dir)

    with open("{0}/{1}".format(root_deployments_dir, "deployment_intent_schema.json"), "w") as text_file:
        text_file.write(intent_template)

    with open("{0}/{1}".format(root_deployments_dir, "sample_utterance.txt"), "w") as text_file:
        text_file.write(utterance_template)

if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python
import os
import random
import sys
import getopt
import string

"""
Script will create a concrete implementation of the Alexa Handler.

"""

handler_file_template = """
from pyalexaskill.AlexaBaseHandler import AlexaBaseHandler


class AlexaDeploymentTestHandler(AlexaBaseHandler):

    # Sample concrete implementation of the AlexaBaseHandler to test the
    # deployment scripts and process.
    # All on_ handlers call the same test response changing the request type
    # spoken.


    def __init__(self, app_id=None):
        super(self.__class__, self).__init__(app_id)


    def _test_response(self, msg):
        session_attributes = {}
        card_title = "Test Response"
        card_output = "Test card output"
        speech_output = "Welcome to the Python Alexa Test Deployment for request type {0}.  It seems to have worked".format(
            msg)
        # If the user either does not reply to the welcome message or says something
        # that is not understood, they will be prompted again with this text.
        reprompt_text = "Reprompt text for the Alexa Test Deployment"
        should_end_session = True

        speechlet = self._build_speechlet_response(card_title, card_output, speech_output, reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)


    def on_processing_error(self, event, context, exc):
        session_attributes = {}
        speech_output = "I am having difficulty fulfilling your request."

        reprompt_text = "I did not hear you"
        should_end_session = True

        if exc:
            speech_output = "I am having difficulty fulfilling your request. {0}".format(exc.message)

        card_output = speech_output
        speechlet = self._build_speechlet_response(self.card_title,
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)


    def on_launch(self, launch_request, session):
        session_attributes = {}
        card_output = "Sample Card Output"
        speech_output = "Sample Speech Output"

        reprompt_text = "I did not hear you"
        should_end_session = False
        speechlet = self._build_speechlet_response(self.card_title,
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)


    def on_session_started(self, session_started_request, session):
        return self._test_response("on session started")


    def on_intent(self, intent_request, session):
        response = None
        session_attributes = {}
        reprompt_text = "I did not hear you sample"
        should_end_session = True
        card_output = "Sample Card Output"
        speech_output = "Sample Speech Output"

        intent_name = self._get_intent_name(intent_request)
        if intent_name == "Your First Intent":
            speechlet = self._build_speechlet_response(self.card_title,
                                                       card_output,
                                                       speech_output,
                                                       reprompt_text,
                                                       should_end_session)

            response = self._build_response(session_attributes, speechlet)

        elif intent_name == "Your Second Intent":
            speechlet = self._build_speechlet_response(self.card_title,
                                                       card_output,
                                                       speech_output,
                                                       reprompt_text,
                                                       should_end_session)

            response = self._build_response(session_attributes, speechlet)
        else:
            raise ValueError("Invalid intent")

        return response

    def on_session_ended(self, session_end_request, session):
        return self._test_response("on session end")

    def on_help_intent(self, intent_request, session):
        session_attributes = {}
        card_output = "Card Help"
        speech_output = "Speech Help"

        reprompt_text = "I did not hear you, {0}".format(speech_output)
        should_end_session = False
        speechlet = self._build_speechlet_response(self.card_title,
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)

    def on_stop_intent(self, intent_request, session):
        return self.on_cancel_intent(intent_request, session)

    def on_cancel_intent(self, intent_request, session):
        session_attributes = {}
        card_output = "Thank you and Good-bye"
        speech_output = "Thank you and Good-bye"

        reprompt_text = "{0}".format(speech_output)
        should_end_session = True
        speechlet = self._build_speechlet_response(self.card_title,
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)

    def on_no_intent(self, intent_request, session):
        return self._test_response("on no intent")

    def on_yes_intent(self, intent_request, session):
        return self._test_response("on yes intent")

    def on_repeat_intent(self, intent_request, session):
        return self._test_response("on repeat intent")

    def on_startover_intent(self, intent_request, session):
        return self._test_response("on start over intent")

"""


def main(argv):
    root_project_dir = ''
    template_file_name = 'AlexaHandler_template_' + ''.join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(5)) + '.py'

    try:
        opts, args = getopt.getopt(argv, "hr:", ["root="])
    except getopt.GetoptError:
        print 'create_alexa_handler.py -r <root project dir>'
        print 'if -r option not supplied it will look for PWD environment variable'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'create_alexa_handler.py -r <root project dir>'
            print 'if -r option not supplied it will look for PWD environment variable'
            sys.exit()
        elif opt in ("-r", "--root"):
            root_project_dir = arg

    if not root_project_dir:
        root_project_dir = os.environ.get("PWD")
        if root_project_dir is None:
            raise ValueError("Must supply -r or --root option")

    with open("{0}/{1}".format(root_project_dir, template_file_name), "w") as text_file:
        text_file.write(handler_file_template)


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python
import os
import random
import sys
import getopt
import string

"""
Script will create an AWS Lambda function deployment.

It expects there to be a deployments directory and it will create a
deployment of the form:

deployment_n

where n is incremented for each deployment based on the existing deployment
directories

If the AWS Lambda function has dependencies those dependencies are expected
to be in the requirements.txt file.

The implementation files are expected to be in the root project directory, and
this command does not currently support deeply nested file structures.

"""

main_file_template = """
import logging
from YourDeploymentHandler import YourDeploymentHandler


#  Main entry point for the Lambda function.
#  In the AWS Lamba console, under the 'Configuration' tab there is an
#  input field called, 'Handler'.  That should be:  main.lambda_handler

#  Handler: main.lambda_handler
#  Role: lambda_basic_execution



logger = logging.getLogger()
logger.setLevel(logging.INFO)


def lambda_handler(event, context):
    logging.info("Executing main lambda_handler for YourDeploymentHandler class")

    deployment_handler = YourDeploymentHandler()
    handler_response = deployment_handler.process_request(event, context)

    return handler_response

"""


def main(argv):
    root_project_dir = ''
    main_file_name = 'main_template_' + ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(5)) + '.py'

    try:
        opts, args = getopt.getopt(argv, "hr:", ["root="])
    except getopt.GetoptError:
        print 'create_aws_main.py -r <root project dir>'
        print 'if -r option not supplied it will look for PWD environment variable'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'create_aws_main.py -r <root project dir>'
            print 'if -r option not supplied it will look for PWD environment variable'
            sys.exit()
        elif opt in ("-r", "--root"):
            root_project_dir = arg

    if not root_project_dir:
        root_project_dir = os.environ.get("PWD")
        if root_project_dir is None:
            raise ValueError("Must supply -r or --root option")

    with open("{0}/{1}".format(root_project_dir, main_file_name), "w") as text_file:
        text_file.write(main_file_template)

if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python
import os
import random
import sys
import getopt
import string

"""
Script will create a concrete implementation of the Alexa Handler.

"""

handler_file_template = """
from pyalexaskill.AlexaAudioBaseHandler import AlexaAudioBaseHandler
import logging
import random


class MyAudioException(Exception):
    pass


class AlexaBwBHandler(AlexaAudioBaseHandler):

    def _build_audio_response(self, speech, title, card):
        session_attributes = {}
        card_title = title
        card_output = card
        speech_output = speech
        # If the user either does not reply to the welcome message or says something
        # that is not understood, they will be prompted again with this text.
        reprompt_text = ""
        should_end_session = True

        speechlet = self._build_speechlet_response(card_title, card_output, speech_output, reprompt_text, should_end_session)

        return self._build_response(session_attributes, speechlet)

    def __init__(self, app_id=None, log_level=logging.INFO):

        self.songs = []
        self.songs.append({
            'name': '',
            'url': '',
            'token': ''
        })
        self.songs.append({
            'name': '',
            'url': '',
            'token': ''
        })
        #add more songs here
        super(AlexaBwBHandler, self).__init__(app_id, log_level)

    def on_system_exceptionencountered(self, event, context):
        try:
            self.logger.error("ERROR: on_system_exceptionencountered: {0}".format(event))
            self.logger.error("ERROR: Type: {0}".format(event['error']['type']))
            self.logger.error("ERROR: Msg: {0}".format(event['error']['message']))
        except:
            pass

        return None

    def on_invalid_response(self, event, context):
        self.logger.error("on_invalid_response: {0}".format(event))
        return self.create_empty_response()

    def on_startover_intent(self, intent_request, session):
        return self._play_song()

    def on_stop_intent(self, intent_request, session):
        self.logger.debug("AlexaBwBHandler.on_stop_intent")
        return self.create_stop_directive()

    def check_app_id(self, event):
        return super(AlexaBwBHandler, self).check_app_id(event)

    def on_processing_error(self, event, context, exc):
        session_attributes = {}
        speech_output = "I am having difficulty fulfilling your request"

        reprompt_text = "I did not hear you, you can ask for train schedules like, ask Chicago Trains when the next train from starting station names goes to ending station name."
        should_end_session = True

        card_output = speech_output
        speechlet = self._build_speechlet_response('BwB Error',
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)


    def on_launchrequest(self, event, context):
        self.logger.debug("Executing on_launch")
        return self._play_song()

    def _enqueue_song(self, current_token):
        song = random.choice(self.songs)
        url = song['url']
        token = song['token']
        self.logger.debug("_enqueue_song: {0}".format(song['name']))
        enqueue_response = self.create_enqueue_directive(current_token, token, url)
        self.logger.debug("Enqueue Response: {0}".format(enqueue_response))
        return enqueue_response

    def _play_song(self, silent=False):
        song = random.choice(self.songs)
        url = song['url']
        token = song['token']
        if silent:
            speech = ""
        else:
            speech = "Playing {0}".format(song['name'])
        card = "Playing {0}".format(song['name'])
        self.logger.debug("_play_song: {0}".format(song['name']))
        session_attributes = {
            'song': song
        }

        play_response = self.create_play_directive(token, url, "REPLACE_ALL", 0, speech, "BwB", card, session_attributes)
        self.logger.debug("Play Response: {0}".format(play_response))
        return play_response

    def on_help_intent(self, intent_request, session):
        session_attributes = {}
        card_output = "You can hear the sizzling sounds of Better with Bacon by saying, play some bacon"
        speech_output = card_output
        card_title = "Better with Bacon"

        reprompt_text = "I did not hear you, {0}".format(speech_output)
        should_end_session = False
        speechlet = self._build_speechlet_response(card_title,
                                                   card_output,
                                                   speech_output,
                                                   reprompt_text,
                                                   should_end_session)

        return self._build_response(session_attributes, speechlet)

    def on_next_intent(self, intent_request, session):
        return self._play_song()

    def on_audioplayer_playbackfailed(self, event, context):
        self.logger.debug("Executing on_audioplayer_playbackfailed_request")
        try:
            self.logger.debug(event)
        except:
            pass

    def on_resume_intent(self, intent_request, session):
        self.logger.debug("Executing on_resume_intent")
        try:
            self.logger.debug(intent_request)
            self.logger.debug(session)
            return self._play_song(silent=True)
        except:
            raise MyAudioException("An error occurred trying to resume audio")


    def on_previous_intent(self, intent_request, session):
        return self._play_song(silent=True)

    def on_shuffleoff_intent(self, intent_request, session):
        return self._build_audio_response("Shuffle off is not yet supported", "BwB", 'Shuffle off is not yet supported')

    def on_shuffleon_intent(self, intent_request, session):
        return self._build_audio_response("Shuffle on is not yet supported", "BwB", 'Shuffle on is not yet supported')

    def on_audioplayer_playbackstarted(self, event, context):
        self.logger.debug("Executing on_audioplayer_playbackstarted_request")
        return None


    def on_loopoff_intent(self, intent_request, session):
        return self._build_audio_response("Loop off is not yet supported", "BwB", 'Loop off is not yet supported')

    def on_playbackcontroller_pausecommandissued(self, event, context):
        try:
            self.logger.debug("Executing on_playbackcontroller_pausecommandissued_request")
            self.logger.debug(event)
        except:
            pass

        return None

    def on_audioplayer_playbackstopped(self, event, context):
        self.logger.debug("Executing on_audioplayer_playbackstopped_request")
        try:
            self.logger.debug("Executing on_audioplayer_playbackstopped_request: event: {0}".format(event))

        except:
            pass

        return None

    def on_pause_intent(self, intent_request, session):
        self.logger.debug("Executing on_pause_intent")
        try:
            self.logger.debug("AlexaBwBHandler.on_pause_intent: intent_request: {0}".format(intent_request))
            self.logger.debug("AlexaBwBHandler.on_pause_intent: session: {0}".format(session))
        except:
            pass

        response = self.create_stop_directive()
        self.logger.debug("on_pause_intent response: {0}".format(response))
        return response

    def on_audioplayer_playbacknearlyfinished(self, event, context):
        self.logger.debug("Executing on_audioplayer_playbacknearlyfinished_request")
        current_token = event['request']['token']
        return self._enqueue_song(current_token)

    def on_loopon_intent(self, intent_request, session):
        return self._build_audio_response("Loop on is not yet supported", "BwB", 'Loop on is not yet supported')

    def on_cancel_intent(self, intent_request, session):
        self.logger.debug("Executing on_cancel_intent")
        return self.create_stop_directive()

    def on_repeat_intent(self, intent_request, session):
        return self._play_song()

    def on_playbaconintent_intent(self, intent_request, session):
        self.logger.debug("Executing on_playbacon_intent")
        return self._play_song()

"""


def main(argv):
    root_project_dir = ''
    template_file_name = 'AlexaHandler_template_' + ''.join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(5)) + '.py'

    try:
        opts, args = getopt.getopt(argv, "hr:", ["root="])
    except getopt.GetoptError:
        print 'create_alexa_handler.py -r <root project dir>'
        print 'if -r option not supplied it will look for PWD environment variable'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'create_alexa_handler.py -r <root project dir>'
            print 'if -r option not supplied it will look for PWD environment variable'
            sys.exit()
        elif opt in ("-r", "--root"):
            root_project_dir = arg

    if not root_project_dir:
        root_project_dir = os.environ.get("PWD")
        if root_project_dir is None:
            raise ValueError("Must supply -r or --root option")

    with open("{0}/{1}".format(root_project_dir, template_file_name), "w") as text_file:
        text_file.write(handler_file_template)


if __name__ == "__main__":
    main(sys.argv[1:])



import logging
from AlexaDeploymentHandler import AlexaDeploymentHandler

"""
Main entry point for the Lambda function.
In the AWS Lamba console, under the 'Configuration' tab there is an
input field called, 'Handler'.  That should be:  main.lambda_handler

Handler: main.lambda_handler
Role: lambda_basic_execution

"""

logger = logging.getLogger()
logger.setLevel(logging.INFO)


def lambda_handler(event, context):
    logging.info("Executing main lambda_handler for AlexaDeploymentHandler class")

    alexa = AlexaDeploymentHandler()
    alexa_response = alexa.process_request(event, context)

    return alexa_response






import abc
import logging
import traceback


class AlexaBaseHandler(object):
    """
    Base class for a python Alexa Skill Set.  Concrete implementations
    are expected to implement the abstract methods.

    See the following for Alexa details:
    https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/handling-requests-sent-by-alexa
    """

    __metaclass__ = abc.ABCMeta

    def __init__(self, app_id=None, log_level=logging.INFO):
        self.logger = logging.getLogger()
        self.logger.setLevel(log_level)
        self.app_id = app_id

    @abc.abstractmethod
    def on_processing_error(self, event, context, exc):
        """
        If an unexpected error occurs during the process_request method
        this handler will be invoked to give the concrete handler
        an opportunity to respond gracefully

        :param exc exception instance
        :return: the output of _build_response
        """
        pass

    def on_intent(self, intent_request, session):
        """
        Implement the IntentRequest.  This is called if there is no specific
        on_<intentname>_intent method.  This gives the implementer the choice
        or writing specific on_intent methods, or calling a single method to
        determine what needs to be called.
        :param intent_request:
        :param session:
        :return: the output of _build_response
        """
        pass


    def check_app_id(self, event):
        """
        Check the App id to make sure it is valid.
        :param event:
        :return: True - app id is valid, False - app id is invalid
        """
        valid_app_id = False
        try:
            if self.app_id and event['session']['application']['applicationId'] != self.app_id:
                valid_app_id = False
            else:
                valid_app_id = True
        except:
            valid_app_id = False

        return valid_app_id

    def _handle_amazon_request(self, event, context):
        """
        Method dynamically calls AMAZON built in requests.  For example, with the
        new Audio Playback, there call requests of the form:
        AudioPlayer.PlaybackStarted
        AudioPlayer.PlaybackFinished
        To keep this generic, this method will dynamically call method of the form:

        A.B
        on_a_b(event, context)
        e.g:
        AudioPlayer.PlaybackStarted will be a method
        on_audioplayer_playbackstarted(..)

        :param event:
        :param context:
        :return:speechlet_response, directive, None
        """
        response = None
        self.logger.debug("_handle_amazon_request: event: {0}".format(event))
        request_type = event['request']['type']
        self.logger.debug("_handle_amazon_request: {0}".format(request_type))
        if request_type:
            parts = request_type.split(".")
            if len(parts) == 1:
                request_type_method_name = "on_{0}".format(parts[0].lower())
            elif len(parts) == 2:
                request_type_method_name = "on_{0}_{1}".format(parts[0].lower(), parts[1].lower())
            else:
                raise NotImplementedError("Unexpected request type: {0}".format(request_type))

            self.logger.debug("_handle_amazon_request: {0}".format(request_type_method_name))
            if hasattr(self, request_type_method_name):
                try:
                    response = getattr(self, request_type_method_name)(event, context)
                except:
                    self.logger.error("Traceback Exception {0}".format(traceback.format_exc()))
                    self.logger.error("ERROR: _handle_amazon_request: {0}".format(request_type_method_name))
                    raise
            else:
                # not every request is required to be implemented - particularly for the
                # playbackcontroller requests
                self.logger.warn("_handle_amazon_request: {0} method not found".format(request_type_method_name))
                #raise NotImplementedError("No method with name: {0} exists in class".format(request_type_method_name))

        return response

    def _handle_amazon_intent(self, event, context):
        """
        Method dynamically calls AMAZON built in intents.
        For example, for the AMAZON.YesIntent, this method will dynamically call
        a method of the form:
        on_yes_intent(intent_request, session)

        This allows for this base class to be extensible to handle the new
        Amazon Alexa Streaming Playback support without having to specially
        look for the streaming intent names.
        :param event:
        :param context:
        :return: speechlet_response
        """
        response = None

        intent_name = self._get_intent_name(event['request'])
        if intent_name is not None and intent_name.startswith("AMAZON."):
            intent_method_name = "on_{0}_intent".format(intent_name.split(".")[1].replace("Intent","").lower())
            self.logger.debug("_handle_amazon_intent: {0}".format(intent_method_name))

            if hasattr(self, intent_method_name):
                try:
                    response = getattr(self, intent_method_name)(event['request'], event['session'])
                except:
                    self.logger.error("Traceback Exception {0}".format(traceback.format_exc()))
                    self.logger.error("ERROR: _handle_amazon_intent: {0}".format(intent_method_name))
                    raise

            else:
                raise NotImplementedError("No method with name: {0} exists in class".format(intent_method_name))

        return response

    def _handle_custom_intent(self, event, context):
        """
        Method dynamically calls customeintents by taking the intent name,
        lower casing it, and prepending the name with on_

        For example, for the MyCustomIntent, this method will dynamically call
        a method of the form:
        on_mycustomintent(intent_request, session)

        If the class has not method with that name, it will look for a
        method with the name, 'on_intent'.

        If no method with that name exists then an error will be thrown.

        :param event:
        :param context:
        :return: speechlet_response
        """
        response = None

        intent_name = self._get_intent_name(event['request'])
        if intent_name is not None:
            intent_method_name = "on_{0}_intent".format(intent_name.lower())
            self.logger.debug("_handle_custom_intent: {0}".format(intent_method_name))

            if hasattr(self, intent_method_name):
                try:
                    response = getattr(self, intent_method_name)(event['request'], event['session'])
                except:
                    self.logger.error("Traceback Exception {0}".format(traceback.format_exc()))
                    self.logger.error("ERROR: _handle_amazon_intent: {0}".format(intent_method_name))
                    raise

            elif hasattr(self, 'on_intent'):
                try:
                    response = getattr(self, 'on_intent')(event['request'], event['session'])
                except:
                    self.logger.error("Traceback Exception {0}".format(traceback.format_exc()))
                    self.logger.error("ERROR: _handle_custom_intent: {0}".format(intent_method_name))
                    raise
            else:
                raise NotImplementedError("No method with name: {0} exists in class".format(intent_method_name))

        return response

    #--------------------------------------------------------
    #--------------- Main Processing Entry Point  -----------
    #--------------------------------------------------------
    def process_request(self, event, context):
        """
        Helper method to process the input Alexa request and
        dispatch to the appropriate on_ handler
        :param event:
        :param context:
        :return: response from the on_ handler
        """
        self.logger.debug("process_request: event: {0}".format(event))

        try:
            request_type = event['request']['type']
            self.logger.debug("event[request][type]: {0}".format(request_type))
        except:
            request_type = None

        try:
            new_session = event['session']['new']
        except:
            new_session = False

        # if its a new session, run the new session code
        try:
            if not self.check_app_id(event):
                raise NotImplementedError("Invalid Application ID")

            response = None
            if new_session is not False and hasattr(self, 'on_session_started'):
                # then it is a new session, and the concrete class has an on_session_started
                # callback
                getattr(self, 'on_session_started')(event['request'], event['session'])

            # regardless of whether its new, handle the request type
            if request_type == "IntentRequest":
                # Only handle IntentRequest here... all others in the else block
                intent_name = self._get_intent_name(event['request'])
                if intent_name is not None and intent_name.startswith("AMAZON."):
                    response = self._handle_amazon_intent(event, context)
                else:
                    # this is a user specific intent, so let the users concrete
                    # implementation handle it.
                    response = self._handle_custom_intent(event, context)
            else:
                # LaunchRequest, SessionEndedRequest, AudioPlayer requests, etc
                # are handled here.
                self.logger.debug("Calling _handle_amazon_request")
                response = self._handle_amazon_request(event, context)

        except Exception as exc:
            self.logger.error("Error in process_request: {0}".format(traceback.format_exc()))
            self.logger.error(exc.message)
            response = self.on_processing_error(event, context, exc)

        return response

    # --------------- Helpers that build all of the responses ----------------------
    def _build_speechlet_response(self, card_title, card_output, speech_output, reprompt_text, should_end_session):
        """
        Internal helper method to build the speechlet portion of the response
        :param card_title:
        :param card_output:
        :param speech_output:
        :param reprompt_text:
        :param should_end_session:
        :return:
        """
        return {
            'outputSpeech': {
                'type': 'PlainText',
                'text': speech_output
            },
            'card': {
                'type': 'Simple',
                'title': card_title,
                'content': card_output
            },
            'reprompt': {
                'outputSpeech': {
                    'type': 'PlainText',
                    'text': reprompt_text
                }
            },
            'shouldEndSession': should_end_session
        }

    def _build_response(self, session_attributes, speechlet_response):
        """
        Internal helper method to build the Alexa response message
        :param session_attributes:
        :param speechlet_response:
        :return: properly formatted Alexa response
        """
        return {
            'version': '1.0',
            'sessionAttributes': session_attributes,
            'response': speechlet_response
        }

    def _is_intent(self, intent_name, intent_request):
        return self._get_intent_name(intent_request) == intent_name

    def _get_intent(self, intent_request):
        if 'intent' in intent_request:
            return intent_request['intent']
        else:
            return None

    def _get_intent_name(self, intent_request):
        intent = self._get_intent(intent_request)
        intent_name = None
        if intent is not None and 'name' in intent:
            intent_name = intent['name']

        return intent_name

    def _slot_exists(self, slot_name, intent_request):
        intent = self._get_intent(intent_request)
        if intent is not None:
            return slot_name in intent['slots']
        else:
            return False

    def _get_slot_value(self, slot_name, intent_request):
        value = None
        try:
            if self._slot_exists(slot_name, intent_request):
                intent = self._get_intent(intent_request)
                value = intent['slots'][slot_name]['value']
            else:
                value = None
        except Exception as exc:
            self.logger.error("Error getting slot value for slot_name={0}".format(slot_name))

        return value



#!/usr/bin/env python
import os
import subprocess
import zipfile
import sys
import getopt
import shutil

"""
Script will create an AWS Lambda function deployment.

It expects there to be a deployments directory and it will create a
deployment of the form:

deployment_n

where n is incremented for each deployment based on the existing deployment
directories

If the AWS Lambda function has dependencies those dependencies are expected
to be in the requirements.txt file.

The implementation files are expected to be in the root project directory, and
this command does not currently support deeply nested file structures.

"""

root_deployments_dir = ''
root_project_dir = ''

# List of files that should be included in the deployment
# Only the files listed here, and the libraries in the requirements.txt
# file will be included in the deployment.
deployment_files = []


def _read_test_requirements():
    filename = "{0}/requirements-test.txt".format(root_project_dir)
    if not os.path.exists(filename):
        return None

    with open(filename, 'r') as f:
        install_requirements = f.readlines()

    return install_requirements


def _read_requirements():
    filename = "{0}/requirements.txt".format(root_project_dir)
    if not os.path.exists(filename):
        return None

    with open(filename, 'r') as f:
        install_requirements = f.readlines()

    return install_requirements


def _get_immediate_subdirectories(a_dir):
    return [name for name in os.listdir(a_dir)
            if os.path.isdir(os.path.join(a_dir, name))]


def _mkdirp(directory):
    if not os.path.isdir(directory):
        os.makedirs(directory)


def _make_deployment_dir():
    _mkdirp(root_deployments_dir)
    all_deployment_directories = _get_immediate_subdirectories(root_deployments_dir)
    max_deployment_number = -1
    for deployment_dir in all_deployment_directories:
        dir_name_elements = deployment_dir.split("_")
        if (len(dir_name_elements) == 2):
            if int(dir_name_elements[1]) > max_deployment_number:
                max_deployment_number = int(dir_name_elements[1])

    if max_deployment_number == -1:
        max_deployment_number = 0

    deployment_name = "deployment_{0}".format(max_deployment_number + 1)
    new_deployment_dir_path = "{0}/{1}".format(root_deployments_dir, deployment_name)

    if not os.path.exists(new_deployment_dir_path):
        os.mkdir(new_deployment_dir_path)

    return (new_deployment_dir_path, deployment_name)

def _install_test_requirements(deployment_requirements, deployment_dir):
    """
    pip install -i https://testpypi.python.org/pypi <requirements line> -t <deployment_dir>
    :param deployment_requirements
    :param deployment_dir:
    :return:
    """
    if os.path.exists(deployment_dir):
        for requirement in deployment_requirements:
            if not requirement.startswith('#'):
                cmd = "pip install -i https://testpypi.python.org/pypi {0} -t {1}".format(requirement, deployment_dir).split()
                return_code = subprocess.call(cmd, shell=False)

def _install_requirements(deployment_requirements, deployment_dir):
    """
    pip install <requirements line> -t <deployment_dir>
    :param deployment_requirements
    :param deployment_dir:
    :return:
    """
    if os.path.exists(deployment_dir):
        for requirement in deployment_requirements:
            if not requirement.startswith('#'):
                cmd = "pip install {0} -t {1}".format(requirement, deployment_dir).split()
                return_code = subprocess.call(cmd, shell=False)


def _copy_deployment_files(deployment_data):
    for item in deployment_data:
        if os.path.exists(item['from']):
            cmd = "cp {0} {1}".format(item['from'], item['to']).split()
            return_code = subprocess.call(cmd, shell=False)
        else:
            raise NameError("Deployment file not found [{0}]".format(item['from']))


def zipdir(dirPath=None, zipFilePath=None, includeDirInZip=False):
    """
    Attribution:  I wish I could remember where I found this on the
    web.  To the unknown sharer of knowledge - thank you.

    Create a zip archive from a directory.

    Note that this function is designed to put files in the zip archive with
    either no parent directory or just one parent directory, so it will trim any
    leading directories in the filesystem paths and not include them inside the
    zip archive paths. This is generally the case when you want to just take a
    directory and make it into a zip file that can be extracted in different
    locations.

    Keyword arguments:

    dirPath -- string path to the directory to archive. This is the only
    required argument. It can be absolute or relative, but only one or zero
    leading directories will be included in the zip archive.

    zipFilePath -- string path to the output zip file. This can be an absolute
    or relative path. If the zip file already exists, it will be updated. If
    not, it will be created. If you want to replace it from scratch, delete it
    prior to calling this function. (default is computed as dirPath + ".zip")

    includeDirInZip -- boolean indicating whether the top level directory should
    be included in the archive or omitted. (default True)

"""
    if not zipFilePath:
        zipFilePath = dirPath + ".zip"
    if not os.path.isdir(dirPath):
        raise OSError("dirPath argument must point to a directory. "
                      "'%s' does not." % dirPath)
    parentDir, dirToZip = os.path.split(dirPath)

    # Little nested function to prepare the proper archive path
    def trimPath(path):
        archivePath = path.replace(parentDir, "", 1)
        if parentDir:
            archivePath = archivePath.replace(os.path.sep, "", 1)
        if not includeDirInZip:
            archivePath = archivePath.replace(dirToZip + os.path.sep, "", 1)
        return os.path.normcase(archivePath)

    outFile = zipfile.ZipFile(zipFilePath, "w",
                              compression=zipfile.ZIP_DEFLATED)
    for (archiveDirPath, dirNames, fileNames) in os.walk(dirPath):
        for fileName in fileNames:
            filePath = os.path.join(archiveDirPath, fileName)
            outFile.write(filePath, trimPath(filePath))
        # Make sure we get empty directories as well
        if not fileNames and not dirNames:
            zipInfo = zipfile.ZipInfo(trimPath(archiveDirPath) + "/")
            # some web sites suggest doing
            # zipInfo.external_attr = 16
            # or
            # zipInfo.external_attr = 48
            # Here to allow for inserting an empty directory.  Still TBD/TODO.
            outFile.writestr(zipInfo, "")
    outFile.close()


def make_target_dirs(target_paths):
    for dirname in set(os.path.dirname(p) for p in target_paths):
        if not os.path.isdir(dirname):
            os.makedirs(dirname)


def main(argv):
    global root_deployments_dir, root_project_dir
    include_files = ''

    try:
        opts, args = getopt.getopt(argv, "hr:i:", ["root=", "include="])
    except getopt.GetoptError:
        print 'create_aws_lambda.py -r <root project dir> -i <include files>'
        print 'if -r option not supplied it will look for PWD environment variable'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'create_aws_lambda.py -r <root project dir> -i <include files>'
            print 'if -r option not supplied it will look for PWD environment variable'
            print '<include files> are relative to root project dir'
            sys.exit()
        elif opt in ("-r", "--root"):
            root_project_dir = arg
        elif opt in ("-i", "--include"):
            include_files = arg

    if not root_project_dir:
        root_project_dir = os.environ.get("PWD")
        if root_project_dir is None:
            raise ValueError("Must supply -r or --root option")

    if not include_files:
        raise ValueError("Must supply -i or --include option")

    root_deployments_dir = "{0}/deployments".format(root_project_dir)
    (deployment_dir, deployment_name) = _make_deployment_dir()

    for include_file in include_files.split(","):
        d = {
            "from": "{0}/{1}".format(root_project_dir, include_file.strip()),
            "to": "{0}/{1}".format(deployment_dir, include_file.strip())
        }
        make_target_dirs([d['to']])
        deployment_files.append(d)

    _copy_deployment_files(deployment_files)

    # standard requirements file
    install_requirements = _read_requirements()
    _install_requirements(install_requirements, deployment_dir)

    # test requirements file (requirements-test.txt)
    install_requirements = _read_test_requirements()
    _install_test_requirements(install_requirements, deployment_dir)


    deployment_num_file_name = "{0}/.deployment_number.txt".format(root_deployments_dir)
    if not os.path.isfile(deployment_num_file_name):
        # first time - be safe and start at deployment 100
        with open(deployment_num_file_name, 'w') as f:
            f.write("100\n")

    next_deployment_number = None
    with open(deployment_num_file_name, 'r+') as f:
        line1 = f.readline()
        try:
            next_deployment_number = int(line1.strip()) + 1
        except:
            next_deployment_number = 1
        while os.path.exists(
                "{0}/deployment_{1}".format(root_deployments_dir, next_deployment_number)) or os.path.exists(
                "{0}/deployment_{1}.zip".format(root_deployments_dir, next_deployment_number)):
            next_deployment_number = next_deployment_number + 1

        f.seek(0)
        f.write("{0}\n".format(next_deployment_number))

    deployment_zip_filename = "{0}/deployment_{1}.zip".format(root_deployments_dir, next_deployment_number)
    zipdir(deployment_dir, deployment_zip_filename)
    shutil.rmtree(deployment_dir, ignore_errors=True)
    print("Created deployment zip file: {0}".format(deployment_zip_filename))

if __name__ == "__main__":
    main(sys.argv[1:])



from pyalexaskill.AlexaBaseHandler import AlexaBaseHandler
import logging


class AlexaAudioBaseHandler(AlexaBaseHandler):
    """
    Base class for a python Alexa Audio Skill Set.  Concrete implementations
    are expected to implement the abstract methods.

    This base class focuses on the additional intents added by the Audio Playback Support.


    See the following for Alexa details:
    https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/handling-requests-sent-by-alexa


    http://stackoverflow.com/questions/39157599/simplest-example-for-streaming-audio-with-alexa/39283692#39283692

    https://developer.amazon.com/public/community/post/Tx1DSINBM8LUNHY/New-Alexa-Skills-Kit-ASK-Feature-Audio-Streaming-in-Alexa-Skills

    https://developer.amazon.com/public/solutions/alexa/alexa-voice-service/reference/audioplayer

    Your skill is not required to respond to AudioPlayer requests but if it does, 
    please be aware that it can only respond with the AudioPlayer directives mentioned 
    earlier (Play, Stop and ClearQueue). The response should not include any of the 
    standard properties such as outputSpeech, just like the AudioPlayer directives.

    """

    def __init__(self, app_id=None, log_level=logging.INFO):
        super(AlexaAudioBaseHandler, self).__init__(app_id, log_level)

    def create_clearqueue_directive(self):
        directive = {
            "version": "1.0",
            "sessionAttributes": {},
            "response": {
                "directives": [
                    {
                        "type": "AudioPlayer.ClearQueue"
                    }
                ],
                "shouldEndSession": True
            }
        }
        return directive

    def create_stop_directive(self):
        stop_response = {
                    "version": "1.0",
                    "sessionAttributes": {},
                    "response": {
                        "directives": [
                            {
                                "type": "AudioPlayer.Stop"
                            }
                        ],
                        "shouldEndSession": True
                    }
                }
        return stop_response

    def create_empty_response(self):
        response = {
            "version": "1.0",
            "sessionAttributes": {},
            "response": {
                "outputSpeech": {},
                "card": {},
                "reprompt": {},
                "shouldEndSession": True
            }
        }
        return response

    def create_enqueue_directive(self, current_token, token, url, session_attributes=None):
        directive = {
            "version": "1.0",
            "sessionAttributes": {},
            "response": {
                "directives": [
                    {
                        "type": "AudioPlayer.Play",
                        "playBehavior": "ENQUEUE",  #"REPLACE_ALL",  #"ENQUEUE", "REPLACE_ENQUEUED"
                        "audioItem": {
                            "stream": {
                                "token": token,
                                "expectedPreviousToken": current_token,
                                "url": url,
                                "offsetInMilliseconds": 0
                            }
                        }
                    }
                ],
                "shouldEndSession": True
            }
        }

        if session_attributes is not None:
            directive['sessionAttributes'] = session_attributes

        return directive

    def create_play_directive(self, token, url, behavior="REPLACE_ALL", offset=0, speech_content=None, card_title=None, card_content=None, session_attributes=None):
        directive = {
            "version": "1.0",
            "sessionAttributes": {},
            "response": {
                "outputSpeech": {},
                "card": {},
                "reprompt": {},
                "directives": [
                    {
                        "type": "AudioPlayer.Play",
                        "playBehavior": behavior,  #"REPLACE_ALL",  #"ENQUEUE", "REPLACE_ENQUEUED"
                        "audioItem": {
                            "stream": {
                                "token": token,
                                "url": url,
                                "offsetInMilliseconds": offset
                            }
                        }
                    }
                ],
                "shouldEndSession": True
            }
        }

        if session_attributes is not None:
            directive['sessionAttributes'] = session_attributes

        if speech_content is not None:
            directive['response']['outputSpeech']['type']="PlainText"
            directive['response']['outputSpeech']['text'] = speech_content

        if card_content is not None:
            directive['response']['card']['type']="Simple"
            directive['response']['card']['title']=card_title
            directive['response']['card']['content'] = card_content

        return directive




import codecs
import os


from setuptools import setup, find_packages

__version__ = "0.2.0"
__title__ = "pyalexa-skill"
__description__ = "Python module with a base class to support an Alexa skill set, and scripts to build an AWS python distibution"
__uri__ = "https://github.com/youngsoul/PyAlexa"
__author__ = "Patrick Ryan"
__email__ = "pat_ryan_99@yahoo.com"
__license__ = "Public Domain"
__copyright__ = "Copyright (c) 2016 Patrick Ryan"

###############################################################################

NAME = "pyalexa-skill"
PACKAGES = find_packages(where=".")
KEYWORDS = ["alexa", "skill"]
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "License :: Public Domain",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Topic :: Software Development :: Libraries :: Python Modules",
]
INSTALL_REQUIRES = []

###############################################################################

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()

if __name__ == "__main__":
    setup(
        name=NAME,
        description=__description__,
        license=__license__,
        url=__uri__,
        version=__version__,
        author=__author__,
        author_email=__email__,
        maintainer=__author__,
        maintainer_email=__email__,
        keywords=KEYWORDS,
        long_description=read("README.rst"),
        packages=PACKAGES,
        package_dir={"": "."},
        zip_safe=False,
        classifiers=CLASSIFIERS,
        install_requires=INSTALL_REQUIRES,
        scripts=['bin/create_aws_lambda.py','bin/create_aws_main.py', 'bin/create_alexa_handler.py', 'bin/create_alexa_test_skills.py','bin/create_alexa_audio_handler.py']
    )